@@ -1,23 +1,24 @@
 import asyncio
+import logging
+import random
 from collections import defaultdict
 from functools import partial
 from itertools import cycle
-import logging
-import random
 
-from dask.optimization import SubgraphCallable
+from tlz import concat, drop, groupby, merge
+
 import dask.config
-from dask.utils import parse_timedelta
-from tlz import merge, concat, groupby, drop
+from dask.optimization import SubgraphCallable
+from dask.utils import parse_timedelta, stringify
 
 from .core import rpc
-from .utils import All, tokey
+from .utils import All
 
 logger = logging.getLogger(__name__)
 
 
 async def gather_from_workers(who_has, rpc, close=True, serializers=None, who=None):
-    """ Gather data directly from peers
+    """Gather data directly from peers
 
     Parameters
     ----------
@@ -76,7 +77,7 @@ async def gather_from_workers(who_has, rpc, close=True, serializers=None, who=No
             for worker, c in coroutines.items():
                 try:
                     r = await c
-                except EnvironmentError:
+                except OSError:
                     missing_workers.add(worker)
                 except ValueError as e:
                     logger.info(
@@ -97,7 +98,7 @@ async def gather_from_workers(who_has, rpc, close=True, serializers=None, who=No
 
 
 class WrappedKey:
-    """ Interface for a key in a dask graph.
+    """Interface for a key in a dask graph.
 
     Subclasses must have .key attribute that refers to a key in a dask graph.
 
@@ -111,14 +112,14 @@ class WrappedKey:
         self.key = key
 
     def __repr__(self):
-        return "%s('%s')" % (type(self).__name__, self.key)
+        return f"{type(self).__name__}('{self.key}')"
 
 
 _round_robin_counter = [0]
 
 
 async def scatter_to_workers(nthreads, data, rpc=rpc, report=True, serializers=None):
-    """ Scatter data directly to workers
+    """Scatter data directly to workers
 
     This distributes data in a round-robin fashion to a set of workers based on
     how many cores they have.  nthreads should be a dictionary mapping worker
@@ -164,7 +165,7 @@ collection_types = (tuple, list, set, frozenset)
 
 
 def unpack_remotedata(o, byte_keys=False, myset=None):
-    """ Unpack WrappedKey objects from collection
+    """Unpack WrappedKey objects from collection
 
     Returns original collection and set of all found WrappedKey objects
 
@@ -210,7 +211,7 @@ def unpack_remotedata(o, byte_keys=False, myset=None):
             if futures:
                 myset.update(futures)
                 futures = (
-                    tuple(tokey(f.key) for f in futures)
+                    tuple(stringify(f.key) for f in futures)
                     if byte_keys
                     else tuple(f.key for f in futures)
                 )
@@ -231,14 +232,13 @@ def unpack_remotedata(o, byte_keys=False, myset=None):
         return typ(outs)
     elif typ is dict:
         if o:
-            values = [unpack_remotedata(v, byte_keys, myset) for v in o.values()]
-            return dict(zip(o.keys(), values))
+            return {k: unpack_remotedata(v, byte_keys, myset) for k, v in o.items()}
         else:
             return o
     elif issubclass(typ, WrappedKey):  # TODO use type is Future
         k = o.key
         if byte_keys:
-            k = tokey(k)
+            k = stringify(k)
         myset.add(o)
         return k
     else:
@@ -246,13 +246,13 @@ def unpack_remotedata(o, byte_keys=False, myset=None):
 
 
 def pack_data(o, d, key_types=object):
-    """ Merge known data into tuple or dict
+    """Merge known data into tuple or dict
 
     Parameters
     ----------
-    o:
+    o
         core data structures containing literals and keys
-    d: dict
+    d : dict
         mapping of keys to data
 
     Examples
@@ -281,13 +281,13 @@ def pack_data(o, d, key_types=object):
 
 
 def subs_multiple(o, d):
-    """ Perform substitutions on a tasks
+    """Perform substitutions on a tasks
 
     Parameters
     ----------
-    o:
+    o
         Core data structures containing literals and keys
-    d: dict
+    d : dict
         Mapping of keys to values
 
     Examples
@@ -312,15 +312,6 @@ def subs_multiple(o, d):
             return o
 
 
-retry_count = dask.config.get("distributed.comm.retry.count")
-retry_delay_min = parse_timedelta(
-    dask.config.get("distributed.comm.retry.delay.min"), default="s"
-)
-retry_delay_max = parse_timedelta(
-    dask.config.get("distributed.comm.retry.delay.max"), default="s"
-)
-
-
 async def retry(
     coro,
     count,
@@ -361,7 +352,7 @@ async def retry(
     Returns
     -------
     Any
-        Whatever `await `coro()` returned
+        Whatever `await coro()` returned
     """
     # this loop is a no-op in case max_retries<=0
     for i_try in range(count):
@@ -372,7 +363,7 @@ async def retry(
             logger.info(
                 f"Retrying {operation} after exception in attempt {i_try}/{count}: {ex}"
             )
-            delay = min(delay_min * (2 ** i_try - 1), delay_max)
+            delay = min(delay_min * (2**i_try - 1), delay_max)
             if jitter_fraction > 0:
                 delay *= 1 + random.random() * jitter_fraction
             await asyncio.sleep(delay)
@@ -383,6 +374,14 @@ async def retry_operation(coro, *args, operation=None, **kwargs):
     """
     Retry an operation using the configuration values for the retry parameters
     """
+
+    retry_count = dask.config.get("distributed.comm.retry.count")
+    retry_delay_min = parse_timedelta(
+        dask.config.get("distributed.comm.retry.delay.min"), default="s"
+    )
+    retry_delay_max = parse_timedelta(
+        dask.config.get("distributed.comm.retry.delay.max"), default="s"
+    )
     return await retry(
         partial(coro, *args, **kwargs),
         count=retry_count,
